unit GWCOMDemo_COMObjectsIn;

interface
uses GWCOMDemo_Base;

Type
  //-------------------------------
  TGWCOMDemo_COMObjectsIn = class(TGWCOMDemo_Base)
  //-------------------------------
  public
    procedure COMObjectIn_Simple(COM_Interface_A: IUnknown);
    procedure COMObjectIn(COM_Interface_A: IUnknown);
    procedure COMObjectInAndOut(COM_Interface_A: IUnknown;
                            out COM_Interface_B: IUnknown);
    procedure COMObjectInOut(out COM_Interface_A: IUnknown);

    procedure Demo_Run_Inner;
    procedure Demo_Run; override;
  end;

//-------------------------------
implementation
//-------------------------------
uses GWMonOLE, GWDemoCode;

//ShowBegin
// Demonstrate giving COM objects to procedures

//---------------------------------------
procedure TGWCOMDemo_COMObjectsIn.COMObjectIn_Simple(COM_Interface_A: IUnknown);
//---------------------------------------
Begin
  Pause('S-010'); // Input parameter is a local ref, hence RefCount is incremented
end;


//---------------------------------------
procedure TGWCOMDemo_COMObjectsIn.COMObjectIn(COM_Interface_A: IUnknown);
//---------------------------------------
Var
  COM_Interface_B: IUnknown;
Begin
  Pause('A-010');  //Input parameter is a local ref,
                   // hence RefCount is incremented

  COM_Interface_B := COM_Interface_A; // We made a new reference
  Pause('A-020');

  COM_Interface_B := nil;  // Remove reference
  Pause('A-030');

  COM_Interface_A := nil;  // Remove input parameter's reference
                           // (same effect as going out of scope at
                           // end of procedure).
  Pause('A-040');
end;

//---------------------------------------
procedure TGWCOMDemo_COMObjectsIn.COMObjectInAndOut(
   {in} COM_Interface_A: IUnknown;
   out  COM_Interface_B: IUnknown);
//---------------------------------------
Begin
  Pause('B-000');  //
  COM_Interface_B := COM_Interface_A;
  Pause('B-010');  //
end;

//---------------------------------------
procedure TGWCOMDemo_COMObjectsIn.COMObjectInOut(out COM_Interface_A: IUnknown);
//---------------------------------------
Begin
  Pause('C-000');  // COM_Interface_A's RefCount got decremented!

  If Assigned(COM_Interface_A) then
    Pause('C-Assigned')
    else Pause('C-UnAssigned');
  // So apparently, an out parameter gets immediately de-assigned on entry to the
  // procedure, making it pointless to pass a COM Object in to an out parameter.

end;

//---------------------------------------
procedure TGWCOMDemo_COMObjectsIn.Demo_Run_Inner;
//---------------------------------------
Var
  COM_Interface_1, COM_Interface_2:  IUnknown;
  Pascal_Object_1:    TGWMonInterfacedObject;
Begin
  Pause('000');

  //----------------------------------------------------
  // Test ordinary create and pass
  //----------------------------------------------------
  COM_Interface_1 := TGWMonInterfacedObject.Create(Mon);
  Pause('010');  // RefCount at 1

  COMObjectIn(COM_Interface_1);
  Pause('020');  // RefCount back at 1

  //----------------------------------------------------
  // Try in + out
  //----------------------------------------------------
  COMObjectInAndOut(COM_Interface_1, COM_Interface_2);
  Pause('050');  // RefCount at 2

  //----------------------------------------------------
  // Try passing COM object IN to an OUT parameter
  //----------------------------------------------------
  COMObjectInOut(COM_Interface_1); // RefCount starting at 2
  Pause('070'); // COM_Interface_1 is now deassigned and RefCount now 1
                // Probably not the expected result!

  //----------------------------------------------------
  // Test passing a Pascal object where a COM interface
  // is expected
  //----------------------------------------------------
  Pascal_Object_1 := TGWMonInterfacedObject.Create(Mon);
  Pause('110');  // RefCount at zero

  COMObjectIn_Simple(Pascal_Object_1);  // proc accepts object and can use, but...
  Pause('120');  // RefCount decremented to zero, so object deleted,
    // so Pascal_Object_1 no longer valid...probably not desired result.
    // So, always make a local COM reference to a COM object before
    // passing it to another procedure as an interface.

end;

//---------------------------------------
procedure TGWCOMDemo_COMObjectsIn.Demo_Run;
//---------------------------------------
Begin
  Pause('Begin');
  Demo_Run_Inner;
  Pause('End');   // Check that all objects are deleted by now!
end;
//ShowEnd

initialization
  DemoCode_Mgr.Add(
    TGWCOMDemo_COMObjectsIn.Create(
      'COM Demo -- Pass COM Objects TO procedures',
      'GWCOMDemo_COMObjectsIn.pas'));
end.

