unit GWRefCT_Main;
{---------------------------------------------------------------
Purpose: Implements demos for various COM object reference-counting
         scenarios.

Author:  Graham Wideman
         graham@wideman-one.com
         www.wideman-one.com

Usage Notes:
------------
To add more COM demos, create a new unit, implementing an object that descends from
TGWCOMDemo_Base.  Use GWCOMDemo_Basic as an example. Be sure to change the
DemoCode_Mgr.Add call in the initialization section appropriately.

By simply including the new unit in the project, its demo routine will show up
on the Demos menu. (ie: no need to "use" the new unit, or to call anything in
the new unit explicitly.)

Along with running this program and stepping through the demo, it may also
be informative to step through the code with the debugger.  If so, simply step
over the Pause procedures.


Revision History
----------------
98-07-20 GW V1.1 original

To-do list
----------
-- The "X" box on title bar attempts to close app and can't if demo is
   running.  Use quit button.
---------------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, StdCtrls, ExtCtrls, Grids,
  GWMonOLE;

type
  tbs_xxx = (tbs_None, tbs_WaitForStart, tbs_Running);

  Tfm_Main = class(TForm)
    Panel1: TPanel;
    MainMenu1: TMainMenu;
    m_File: TMenuItem;
    m_Demos: TMenuItem;
    mi_Quit: TMenuItem;
    lb_Demo: TLabel;
    bt_Step: TButton;
    mm_SourceCode: TMemo;
    bt_Start: TButton;
    bt_End: TButton;
    bt_MonDock: TButton;
    mi_About: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure mi_QuitClick(Sender: TObject);
    procedure bt_StepClick(Sender: TObject);
    procedure mm_SourceCodeEnter(Sender: TObject);
    procedure bt_StartClick(Sender: TObject);
    procedure bt_EndClick(Sender: TObject);
    procedure bt_MonDockClick(Sender: TObject);
    procedure mi_AboutClick(Sender: TObject);
  private
    { Private declarations }

    procedure MenuItemClickCallback(Sender: TObject);
    procedure SetButtonState(AState: tbs_xxx);
  public
   { Public declarations }
    Ffm_OLEMonitor: Tfm_OLEMonitor;
  end;

var
  fm_Main: Tfm_Main;

implementation
{$R *.DFM}
  uses fm_About, GWDemoCode;

//-------------------------------------------------
procedure Tfm_Main.FormCreate(Sender: TObject);
//-------------------------------------------------
begin
  DemoCode_Mgr.Init(m_Demos, MenuItemClickCallback, lb_Demo, mm_SourceCode);
  SetButtonState(tbs_None);
  Top := 100;
  Left := 100;
  Ffm_OLEMonitor := Tfm_OLEMonitor.Create(self);
end;

//-------------------------------------------------
procedure Tfm_Main.mi_QuitClick(Sender: TObject);
//-------------------------------------------------
begin
  DemoCode_Mgr.EndFlag := True;  // so we can actually escape
  Close;
end;

//-------------------------------------------------
procedure Tfm_Main.MenuItemClickCallback(Sender: TObject);
//-------------------------------------------------
Var
  DemoNum: integer;
Begin
  SetButtonState(tbs_None);
  DemoNum := DemoCode_Mgr.Demo_Load(Sender);
  If DemoNum <> -1 then SetButtonState(tbs_WaitForStart);
end;

//-------------------------------------------------
procedure Tfm_Main.bt_StepClick(Sender: TObject);
//-------------------------------------------------
begin
  DemoCode_Mgr.Step;
end;

//-------------------------------------------------
procedure Tfm_Main.mm_SourceCodeEnter(Sender: TObject);
//-------------------------------------------------
begin
  bt_MonDock.SetFocus;
end;

//-------------------------------------------------
procedure Tfm_Main.bt_StartClick(Sender: TObject);
//-------------------------------------------------
begin
  SetButtonState(tbs_Running);
  DemoCode_Mgr.Demo_Run;
  SetButtonState(tbs_WaitForStart);
end;

//-------------------------------------------------
procedure Tfm_Main.SetButtonState(AState: tbs_xxx);
//-------------------------------------------------
Begin
  Case AState of
    tbs_None: Begin
      bt_Start.Enabled := false;
      bt_Step.Enabled  := false;
      bt_End.Enabled   := false;
    end;
    tbs_WaitForStart: Begin
      bt_Start.Enabled := true;
      bt_Step.Enabled  := false;
      bt_End.Enabled   := false;
    end;
    tbs_Running: Begin
      bt_Start.Enabled := false;
      bt_Step.Enabled  := true;
      bt_End.Enabled   := true;
    end;
  end; { case }


end;

//-------------------------------------------------
procedure Tfm_Main.bt_EndClick(Sender: TObject);
//-------------------------------------------------
begin
  DemoCode_Mgr.EndFlag := true;
end;

procedure Tfm_Main.bt_MonDockClick(Sender: TObject);
begin
  Ffm_OLEMonitor.Show;
  Ffm_OLEMonitor.DockMe;
end;

//-------------------------------------------------
procedure Tfm_Main.mi_AboutClick(Sender: TObject);
//-------------------------------------------------
begin

  fm_AboutBox.Left := Left+ 100;
  fm_AboutBox.Top  := Top + 100;
  fm_AboutBox.ShowModal;
end;

end.


